@@ -7,6 +7,10 @@ PROPERTY_TARGET_TANK_TEMPERATURE = "target_tank_temperature"
 PROPERTY_OPERATION_MODE = "operation_mode"
 PROPERTY_ZONE_1_TARGET_TEMPERATURE = "zone_1_target_temperature"
 PROPERTY_ZONE_2_TARGET_TEMPERATURE = "zone_2_target_temperature"
+PROPERTY_ZONE_1_TARGET_HEAT_FLOW_TEMPERATURE = "zone_1_target_heat_flow_temperature"
+PROPERTY_ZONE_2_TARGET_HEAT_FLOW_TEMPERATURE = "zone_2_target_heat_flow_temperature"
+PROPERTY_ZONE_1_TARGET_COOL_FLOW_TEMPERATURE = "zone_1_target_heat_cool_temperature"
+PROPERTY_ZONE_2_TARGET_COOL_FLOW_TEMPERATURE = "zone_2_target_heat_cool_temperature"
 PROPERTY_ZONE_1_OPERATION_MODE = "zone_1_operation_mode"
 PROPERTY_ZONE_2_OPERATION_MODE = "zone_2_operation_mode"
 
@@ -144,7 +148,7 @@ class Zone:
         This value is not available in the standard state poll response. The poll
         update frequency can be a little bit lower that expected.
         """
-        return self._device_conf()["Device"]["FlowTemperature"]
+        return self._device_conf()["Device"][f"FlowTemperatureZone{self.zone_index}"]
 
     @property
     def return_temperature(self) -> float:
@@ -153,7 +157,7 @@ class Zone:
         This value is not available in the standard state poll response. The poll
         update frequency can be a little bit lower that expected.
         """
-        return self._device_conf()["Device"]["ReturnTemperature"]
+        return self._device_conf()["Device"][f"ReturnTemperatureZone{self.zone_index}"]
 
     @property
     def target_flow_temperature(self) -> Optional[float]:
@@ -167,6 +171,24 @@ class Zone:
 
         return state.get(f"SetCoolFlowTemperatureZone{self.zone_index}")
 
+    async def set_target_flow_temperature(self, target_flow_temperature):
+        """Set target flow temperature of this zone."""
+        op_mode = self.operation_mode
+        if op_mode is None:
+            return None
+
+        if self.zone_index == 1:
+            if self.operation_mode == ZONE_OPERATION_MODE_HEAT:
+                prop = PROPERTY_ZONE_1_TARGET_HEAT_FLOW_TEMPERATURE
+            else:
+                prop = PROPERTY_ZONE_1_TARGET_COOL_FLOW_TEMPERATURE
+        else:
+            if self.operation_mode == ZONE_OPERATION_MODE_HEAT:
+                prop = PROPERTY_ZONE_2_TARGET_HEAT_FLOW_TEMPERATURE
+            else:
+                prop = PROPERTY_ZONE_2_TARGET_COOL_FLOW_TEMPERATURE
+        await self._device.set({prop: target_flow_temperature})
+
     @property
     def operation_mode(self) -> Optional[str]:
         """Return current operation mode.
@@ -180,12 +202,8 @@ class Zone:
         if state is None:
             return None
 
-        mode = state.get(f"OperationModeZone{self.zone_index}")
-        if not isinstance(mode, int):
-            raise ValueError(f"Invalid operation mode [{mode}]")
-
         return _ZONE_OPERATION_MODE_LOOKUP.get(
-            mode,
+            state.get(f"OperationModeZone{self.zone_index}"),
             ZONE_OPERATION_MODE_UNKNOWN,
         )
 
@@ -220,17 +238,29 @@ class AtwDevice(Device):
         flags = state.get(EFFECTIVE_FLAGS, 0)
 
         if key == PROPERTY_TARGET_TANK_TEMPERATURE:
-            state["SetTankWaterTemperature"] = self.round_temperature(value)
+            state["SetTankWaterTemperature"] = value
             flags = flags | 0x1000000000020
         elif key == PROPERTY_OPERATION_MODE:
             state["ForcedHotWaterMode"] = value == OPERATION_MODE_FORCE_HOT_WATER
             flags = flags | 0x10000
         elif key == PROPERTY_ZONE_1_TARGET_TEMPERATURE:
-            state["SetTemperatureZone1"] = self.round_temperature(value)
+            state["SetTemperatureZone1"] = value
             flags = flags | 0x200000080
         elif key == PROPERTY_ZONE_2_TARGET_TEMPERATURE:
-            state["SetTemperatureZone2"] = self.round_temperature(value)
+            state["SetTemperatureZone2"] = value
             flags = flags | 0x800000200
+        elif key == PROPERTY_ZONE_1_TARGET_HEAT_FLOW_TEMPERATURE:
+            state["SetHeatFlowTemperatureZone1"] = value
+            flags = flags | 0x1000000000000
+        elif key == PROPERTY_ZONE_1_TARGET_COOL_FLOW_TEMPERATURE:
+            state["SetCoolFlowTemperatureZone1"] = value
+            flags = flags | 0x1000000000000
+        elif key == PROPERTY_ZONE_2_TARGET_HEAT_FLOW_TEMPERATURE:
+            state["SetHeatFlowTemperatureZone2"] = value
+            flags = flags | 0x1000000000000
+        elif key == PROPERTY_ZONE_2_TARGET_COOL_FLOW_TEMPERATURE:
+            state["SetCoolFlowTemperatureZone2"] = value
+            flags = flags | 0x1000000000000
         elif key == PROPERTY_ZONE_1_OPERATION_MODE:
             # Captures required to implement
             # 0x08
